@@ -1,6 +1,4 @@
 import json
-import copy
-
 import snappi
 from snappi_ixnetwork.exceptions import SnappiIxnException
 from snappi_ixnetwork.timer import Timer
@@ -343,7 +341,6 @@ class TrafficItem(CustomField):
         imports["xpath"] = "/"
         href = "%sresourceManager" % self._api._ixnetwork.href
         url = "%s/operations/importconfig" % href
-        import json
 
         payload = {
             "arg1": href,
@@ -402,7 +399,7 @@ class TrafficItem(CustomField):
             return {}
         paths = {}
         for i, dev_name in enumerate(dev_names):
-            paths[dev_name] = {"dev_info": self._api.get_ixn_object(dev_name)}
+            paths[dev_name] = {"dev_info": self._api.ixn_objects.get(dev_name)}
             paths[dev_name]["type"] = self._api.get_device_encap(dev_name)
         return paths
 
@@ -457,28 +454,29 @@ class TrafficItem(CustomField):
             name = names[0]
             dev_info = devices[name]["dev_info"]
             xpath = dev_info.xpath
-            if xpath in self._api.compacted_ref:
-                cmp_names = set(self._api.compacted_ref[xpath])
+            cmp_names = set(dev_info.names)
+            if len(cmp_names) > 0:
                 inter_names = cmp_names.intersection(set(names))
                 # todo: optimize within scalable
                 if len(inter_names) == len(cmp_names):
                     endpoints.append(xpath)
                     gen_name = inter_names
                 else:
-                    gen_name = set([name])
+                    gen_name = name
                     scalable_endpoints.append(
                         {
-                            "arg1": dev_info.xpath,
+                            "arg1": xpath,
                             "arg2": 1,
                             "arg3": 1,
-                            "arg4": dev_info.index,
+                            "arg4": dev_info.index + 1,
                             "arg5": dev_info.multiplier,
                         }
                     )
-
             else:
-                gen_name = set([name])
+                gen_name = name
                 endpoints.append(xpath)
+            if not isinstance(gen_name, set):
+                gen_name = {gen_name}
             names = list(set(names).difference(gen_name))
 
     def create_traffic(self, config):
@@ -914,9 +912,6 @@ class TrafficItem(CustomField):
                 field_json["value"] = "0001"
         field_json["activeFieldChoice"] = active_field
         field_json["auto"] = False if choice != "auto" else True
-        # TODO: Remove the below for snappi 6.x
-        if snappi_field.__class__.__name__ == "PatternFlowIpv4Protocol":
-            field_json["auto"] = True
         return
 
     def _set_default(self, ixn_field, field_choice):
@@ -1069,7 +1064,7 @@ class TrafficItem(CustomField):
 
     def transmit(self, request):
         """Set flow transmit
-        1) If start then start any device protocols that are traffic dependent
+        1) check set_protocol_state for device protocols
         2) If start then generate and apply traffic
         3) Execute requested transmit action (start|stop|pause|resume)
         """
@@ -1084,9 +1079,10 @@ class TrafficItem(CustomField):
 
         if request.state == "start":
             if len(self._api._topology.find()) > 0:
-                with Timer(self._api, "Devices start"):
-                    self._api._ixnetwork.StartAllProtocols("sync")
-                    self._api.check_protocol_statistics()
+                glob_topo = self._api._globals.Topology.refresh()
+                if glob_topo.Status == "notStarted":
+                    raise Exception("Please start protocols using set_protocol_state "
+                                    "before start traffic")
             if len(self._api._traffic_item.find()) == 0:
                 return
             self._api._traffic_item.find(State="^unapplied$")
@@ -1127,10 +1123,6 @@ class TrafficItem(CustomField):
                 if len(self._api._traffic_item) > 0:
                     with Timer(self._api, "Flows pause"):
                         self._api._traffic_item.PauseStatelessTraffic(True)
-        if request.state == "stop":
-            if len(self._api._topology.find()) > 0:
-                with Timer(self._api, "Devices stop"):
-                    self._api._ixnetwork.StopAllProtocols("sync")
 
     def _set_result_value(
         self, row, column_name, column_value, column_type=str
@@ -1254,100 +1246,47 @@ class TrafficItem(CustomField):
                         self._set_result_value(
                             flow_row, external_name, 0, external_type
                         )
-                    if self._api._flow_tracking:
-                        flow_rows[
-                            traffic_item["name"]
-                            + stream["txPortName"]
-                            + rx_port_name
-                        ] = flow_row
-                    else:
-                        flow_rows[traffic_item["name"]] = flow_row
-        if self._api._flow_tracking:
-            table = self._api.assistant.StatViewAssistant("Flow Statistics")
-            for row in table.Rows:
-                name = row["Traffic Item"]
+                    flow_rows[
+                        traffic_item["name"]
+                    ] = flow_row
+
+        flow_stat = self._api.assistant.StatViewAssistant("Traffic Item Statistics")
+        for row in flow_stat.Rows:
+            name = row["Traffic Item"]
+            if len(flow_names) > 0 and name not in flow_names:
+                continue
+            if name in flow_rows:
+                flow_row = flow_rows[name]
                 if (
-                    len(flow_names) > 0
-                    and row["Traffic Item"] not in flow_names
+                    float(row["Tx Frame Rate"]) > 0
+                    or int(row["Tx Frames"]) == 0
                 ):
-                    continue
-                if (
-                    row["Traffic Item"] + row["Tx Port"] + row["Rx Port"]
-                    in flow_rows
-                ):
-                    flow_row = flow_rows[
-                        row["Traffic Item"] + row["Tx Port"] + row["Rx Port"]
-                    ]
-                    if (
-                        float(row["Tx Frame Rate"]) > 0
-                        or int(row["Tx Frames"]) == 0
-                    ):
-                        flow_row["transmit"] = "started"
-                    else:
-                        flow_row["transmit"] = "stopped"
-                    for (
-                        external_name,
-                        internal_name,
-                        external_type,
-                    ) in self._RESULT_COLUMNS:
-                        # keep plugging values for next columns even if the
-                        # current one raises exception
-                        try:
-                            self._set_result_value(
-                                flow_row,
-                                external_name,
-                                row[internal_name],
-                                external_type,
-                            )
-                        except Exception:
-                            # TODO print a warning maybe ?
-                            pass
-                    if name in self.flows_has_latency:
-                        self._construct_latency(flow_row, row)
-                    if name in self.flows_has_timestamp:
-                        self._construct_timestamp(flow_row, row)
-                    if name not in self.flows_has_loss:
-                        flow_row.pop("loss")
-        else:
-            flow_stat = self._api.assistant.StatViewAssistant(
-                "Traffic Item Statistics"
-            )
-            for row in flow_stat.Rows:
-                name = row["Traffic Item"]
-                if len(flow_names) > 0 and name not in flow_names:
-                    continue
-                if name in flow_rows:
-                    flow_row = flow_rows[name]
-                    if (
-                        float(row["Tx Frame Rate"]) > 0
-                        or int(row["Tx Frames"]) == 0
-                    ):
-                        flow_row["transmit"] = "started"
-                    else:
-                        flow_row["transmit"] = "stopped"
-                    for (
-                        external_name,
-                        internal_name,
-                        external_type,
-                    ) in self._RESULT_COLUMNS:
-                        # keep plugging values for next columns even if the
-                        # current one raises exception
-                        try:
-                            self._set_result_value(
-                                flow_row,
-                                external_name,
-                                row[internal_name],
-                                external_type,
-                            )
-                        except Exception:
-                            # TODO print a warning maybe ?
-                            pass
-                    if name in self.flows_has_latency:
-                        self._construct_latency(flow_row, row)
-                    if name in self.flows_has_timestamp:
-                        self._construct_timestamp(flow_row, row)
-                    if name not in self.flows_has_loss:
-                        flow_row.pop("loss")
+                    flow_row["transmit"] = "started"
+                else:
+                    flow_row["transmit"] = "stopped"
+                for (
+                    external_name,
+                    internal_name,
+                    external_type,
+                ) in self._RESULT_COLUMNS:
+                    # keep plugging values for next columns even if the
+                    # current one raises exception
+                    try:
+                        self._set_result_value(
+                            flow_row,
+                            external_name,
+                            row[internal_name],
+                            external_type,
+                        )
+                    except Exception:
+                        # TODO print a warning maybe ?
+                        pass
+                if name in self.flows_has_latency:
+                    self._construct_latency(flow_row, row)
+                if name in self.flows_has_timestamp:
+                    self._construct_timestamp(flow_row, row)
+                if name not in self.flows_has_loss:
+                    flow_row.pop("loss")
         return list(flow_rows.values())
 
     def _construct_latency(self, flow_row, row):
